/*------------------------------------------------------------------------------
 *
 * Calculate_Compensator() - Called from Menu_B() -
 *                           1.  Determines the Epsilon and Omega refractive
 *                               indices for quartz, calcite, or MAG at the
 *                               given wavelength depending on the value of
 *                               Mchoice.
 *                           2.  Calls Report_Error() if input data will produce
 *                               zero denominators or negative square roots.
 *                           3.  Determines the retardation of the the
 *                               compensator using the Berek or Ehringhaus
 *                               equation depending on the value of Bchoice.
 *                           4.  Diplays compensator plate retardation at the
 *                               given wavelength to the screen.  The display is
 *                               is stopped from scrolling to allow the user to
 *                               read the results - the return key must be
 *                               pressed to continue.
 *
 *------------------------------------------------------------------------------
 */

#include "opt_cal.h"
Calculate_Compensator(Thick,Lambda,T_angle,Bchoice,Mchoice)
double Thick;          /* compensator plate thickness */
double Lambda;         /* wavelength of light         */
double T_angle;        /* compensator tilt angle      */
char Bchoice;          /* compensator equation choice */
char Mchoice;          /* compensator material choice */
{
  char x;              /* dummy scanning variable     */

  double Omega;        /* a refractive index of the selected material */
  double Epsilon;      /* a refractive index of the selected material */
  double Delta;        /* compensator retardation                     */

  int ErrorFlag = 0;
  double Radian = 57.29577951;   /* conversion for degrees to radians */

  extern double sqr();
  extern Report_Error();

  double qw1 = 0.71966632;       /* constants for determining the Omega and */
  double qw2 = 1.7111015E-5;     /* Epsilon indices of QUARTZ.              */
  double qw3 = 7.64024834;
  double qw4 = -7134.65530769;
  double qe1 = 0.70491170;
  double qe2 = 1.7335126E-5;
  double qe3 = 7.77382380;
  double qe4 = -7144.89756715;

  double cw1 = 0.56943676;       /* constants for determining the Omega and */
  double cw2 = 1.6530905E-5;     /* Epsilon indices of CALCITE.             */
  double cw3 = 10.87078737;
  double cw4 = -9111.74306080;
  double ce1 = 0.83572236;
  double ce2 = 6.4330579E-6;
  double ce3 = 4.93666362;
  double ce4 = -7305.53330349;

  double mw1 = 1.11374410;       /* constants for determining the Omega and */
  double mw2 = 1.4344653E-5;     /* Epsilon indices of MAG.                 */
  double mw3 = 6.51423376;
  double mw4 = -6874.44695109;
  double me1 = 1.07436028;
  double me2 = 1.4362171E-5;
  double me3 = 6.41284774;
  double me4 = -6720.96676898;

  switch(Mchoice)
  {
    case '1': Omega   = sqrt( 1 /
                              (qw1 + qw2*Lambda + qw3/Lambda + qw4/sqr(Lambda)) +
                              1
                            );
              Epsilon = sqrt( 1 /
                              (qe1 + qe2*Lambda + qe3/Lambda + qe4/sqr(Lambda)) +
                              1
                            );
              break;
    case '2': Omega   = sqrt( 1 /
                              (cw1 + cw2*Lambda + cw3/Lambda + cw4/sqr(Lambda)) +
                              1
                            );
              Epsilon = sqrt( 1 /
                              (ce1 + ce2*Lambda + ce3/Lambda + ce4/sqr(Lambda)) +
                              1
                            );
              break;

    case '3': Omega   = sqrt( 1 /
                              (mw1 + mw2*Lambda + mw3/Lambda + mw4/sqr(Lambda)) +
                              1
                            );
              Epsilon = sqrt( 1 /
                              (me1 + me2*Lambda + me3/Lambda + me4/sqr(Lambda)) +
                              1
                            );
              break;
  }/*END SWITCH - Mchoice*/
  switch(Bchoice)
  {
    case '1': if( 1-sqr(sin(T_angle/Radian))/sqr(Epsilon) < 0 ||
                  1-sqr(sin(T_angle/Radian))/sqr(Omega)   < 0  )
               {
                 Report_Error(19);
                 ErrorFlag = 1;
               }

              else
               {
                 if(Mchoice == '2')
                   Delta = 1e6 * Thick * Omega *
                           ( sqrt( 1-sqr(sin(T_angle/Radian))/sqr(Omega)   ) -
                             sqrt( 1-sqr(sin(T_angle/Radian))/sqr(Epsilon) )
                           );
                 else
                   Delta = 1e6 * Thick * Omega *
                           ( sqrt( 1-sqr(sin(T_angle/Radian))/sqr(Epsilon) ) -
                             sqrt( 1-sqr(sin(T_angle/Radian))/sqr(Omega)   )
                           );
                }
              break;
    case '2': if( sqr(Epsilon) - sqr(sin(T_angle/Radian)) < 0 ||
                  sqr(Omega)   - sqr(sin(T_angle))        < 0  )
               {
                 Report_Error(19);
                 ErrorFlag = 1;
               }

              else
               {
                 if(Mchoice == '2')
                   Delta = 1e6 * Thick *
                           ( (Epsilon/Omega) *
                             sqrt( sqr(Omega)   - sqr(sin(T_angle/Radian)) ) -
                             sqrt( sqr(Epsilon) - sqr(sin(T_angle/Radian)) )
                           );
                 else
                   Delta = 1e6 * Thick *
                           ( sqrt( sqr(Epsilon) - sqr(sin(T_angle/Radian)) ) -
                             (Epsilon/Omega) *
                             sqrt( sqr(Omega)   - sqr(sin(T_angle/Radian)) )
                           );
               }
              break;

  }/*END SWITCH*/


  if(ErrorFlag == 0)
  {
    printf("\n\tRetardation = %.1f nm at %.1f lambda\n", Delta, Lambda);

    if( (Bchoice == '1' && Mchoice != '3') ||
        (Bchoice == '2' && Mchoice == '3') )
     {
       printf("\n\tI do not have access to tables for your selection of\n");
       printf("\tmaterial and compensator type.  The results the program\n");
       printf("\tprints should be checked with your manual.  Please send me\n");
       printf("\ta copy of your manual so I can check the program in more\n");
       printf("\tdepth.\n");
     }

    fflush(stdin);
    printf("\nPress 'Return' to continue\n\n");
    scanf("%c", &x);
  }

}/*END Calculate_Compensator*/
/*-----------------------------------------------------------------------------*/
